@@ -493,7 +493,7 @@ int GWENHYWFAR_CB GWEN_Crypt_TokenFile_Create(GWEN_CRYPT_TOKEN *ct, uint32_t gid
 
 
 
-int GWENHYWFAR_CB GWEN_Crypt_TokenFile_Open(GWEN_CRYPT_TOKEN *ct, int admin, uint32_t gid){
+int GWENHYWFAR_CB GWEN_Crypt_TokenFile_Open(GWEN_CRYPT_TOKEN *ct, GWEN_UNUSED int admin, uint32_t gid){
   GWEN_CRYPT_TOKEN_FILE *lct;
   int rv;
 
@@ -1190,12 +1190,14 @@ GWEN_Crypt_TokenFile__Sign(GWEN_CRYPT_TOKEN *ct,
   GWEN_BUFFER *srcBuf;
   int i;
   int rv;
+  GWEN_CRYPT_PADDALGOID aid;
 
   assert(ct);
   lct=GWEN_INHERIT_GETDATA(GWEN_CRYPT_TOKEN, GWEN_CRYPT_TOKEN_FILE, ct);
   assert(lct);
 
   DBG_INFO(GWEN_LOGDOMAIN, "Signing with key %d", keyId);
+  aid=GWEN_Crypt_PaddAlgo_GetId(a);
 
   /* reload if needed */
   rv=GWEN_Crypt_TokenFile__ReloadIfNeeded(ct, gid);
@@ -1241,14 +1243,83 @@ GWEN_Crypt_TokenFile__Sign(GWEN_CRYPT_TOKEN *ct,
 
   /* copy to a buffer for padding */
   srcBuf=GWEN_Buffer_new(0, inLen, 0, 0);
-  GWEN_Buffer_AppendBytes(srcBuf, (const char*)pInData, inLen);
 
-  /* padd according to given algo */
-  rv=GWEN_Padd_ApplyPaddAlgo(a, srcBuf);
-  if (rv) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Buffer_free(srcBuf);
-    return rv;
+  if (aid==GWEN_Crypt_PaddAlgoId_Pkcs1_Pss_Sha256) {
+    const GWEN_CRYPT_TOKEN_KEYINFO *ki;
+    int nbits;
+    const uint8_t *modPtr;
+    uint32_t modLen;
+    GWEN_MDIGEST *md;
+
+    switch(keyId & 0xffff) {
+    case 1:  ki=GWEN_CTF_Context_GetLocalSignKeyInfo(ctx); break;
+    case 5:  ki=GWEN_CTF_Context_GetLocalAuthKeyInfo(ctx); break;
+    default: ki=NULL;
+    }
+
+    if (ki==NULL) {
+      DBG_ERROR(GWEN_LOGDOMAIN, "No information for key %d", keyNum);
+      GWEN_Buffer_free(srcBuf);
+      return GWEN_ERROR_GENERIC;
+    }
+
+    /* calculate real number of bits */
+    modPtr=GWEN_Crypt_Token_KeyInfo_GetModulusData(ki);
+    modLen=GWEN_Crypt_Token_KeyInfo_GetModulusLen(ki);
+    nbits=modLen*8;
+    while(modLen && *modPtr==0) {
+      nbits-=8;
+      modLen--;
+      modPtr++;
+    }
+    if (modLen) {
+      uint8_t b=*modPtr;
+      int i;
+      uint8_t mask=0x80;
+
+      for (i=0; i<8; i++) {
+	if (b & mask)
+	  break;
+	nbits--;
+	mask>>=1;
+      }
+    }
+
+    if (nbits==0) {
+      DBG_ERROR(GWEN_LOGDOMAIN, "Empty modulus");
+      GWEN_Buffer_free(srcBuf);
+      return GWEN_ERROR_GENERIC;
+    }
+
+    md=GWEN_MDigest_Sha256_new();
+    GWEN_Buffer_AllocRoom(srcBuf, modLen);
+
+    rv=GWEN_Padd_AddPkcs1Pss((uint8_t*) GWEN_Buffer_GetStart(srcBuf),
+			     GWEN_Buffer_GetMaxUnsegmentedWrite(srcBuf),
+			     nbits,
+			     pInData, inLen,
+			     inLen,
+			     md);
+    GWEN_MDigest_free(md);
+    if (rv<0) {
+      DBG_ERROR(GWEN_LOGDOMAIN, "here (%d)", rv);
+      GWEN_Buffer_free(srcBuf);
+      return rv;
+    }
+
+    GWEN_Buffer_IncrementPos(srcBuf, rv);
+    GWEN_Buffer_AdjustUsedBytes(srcBuf);
+  }
+  else {
+    GWEN_Buffer_AppendBytes(srcBuf, (const char*)pInData, inLen);
+
+    /* padd according to given algo */
+    rv=GWEN_Padd_ApplyPaddAlgo(a, srcBuf);
+    if (rv) {
+      DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+      GWEN_Buffer_free(srcBuf);
+      return rv;
+    }
   }
 
   /* sign with key */
@@ -1360,13 +1431,14 @@ GWEN_Crypt_TokenFile__Verify(GWEN_CRYPT_TOKEN *ct,
   }
 
   if (aid==GWEN_Crypt_PaddAlgoId_Iso9796_2 ||
-      aid==GWEN_Crypt_PaddAlgoId_Pkcs1_2) {
+	   aid==GWEN_Crypt_PaddAlgoId_Pkcs1_2 ||
+	   aid==GWEN_Crypt_PaddAlgoId_Pkcs1_Pss_Sha256) {
     GWEN_BUFFER *tbuf;
     uint32_t l;
 
     /* these algos add random numbers, we must use encrypt fn here and
      * compare the decrypted and unpadded data with the source data */
-    tbuf=GWEN_Buffer_new(0, inLen+16, 0, 0);
+    tbuf=GWEN_Buffer_new(0, signatureLen+16, 0, 0);
     l=GWEN_Buffer_GetMaxUnsegmentedWrite(tbuf);
     rv=GWEN_Crypt_Key_Encipher(k,
 			       pSignatureData, signatureLen,
@@ -1380,21 +1452,83 @@ GWEN_Crypt_TokenFile__Verify(GWEN_CRYPT_TOKEN *ct,
     GWEN_Buffer_IncrementPos(tbuf, l);
     GWEN_Buffer_AdjustUsedBytes(tbuf);
 
-    rv=GWEN_Padd_UnapplyPaddAlgo(a, tbuf);
-    if (rv<0) {
-      DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-      GWEN_Buffer_free(tbuf);
-      return rv;
+    if (aid==GWEN_Crypt_PaddAlgoId_Pkcs1_Pss_Sha256) {
+      const GWEN_CRYPT_TOKEN_KEYINFO *ki;
+      int nbits;
+      const uint8_t *modPtr;
+      uint32_t modLen;
+      GWEN_MDIGEST *md;
+
+      if (keyNum==3)
+	ki=GWEN_CTF_Context_GetRemoteSignKeyInfo(ctx);
+      else
+	ki=GWEN_CTF_Context_GetRemoteAuthKeyInfo(ctx);
+      if (ki==NULL) {
+	DBG_ERROR(GWEN_LOGDOMAIN, "No information for key %d", keyNum);
+	GWEN_Buffer_free(tbuf);
+        return GWEN_ERROR_GENERIC;
+      }
+
+      /* calculate real number of bits */
+      modPtr=GWEN_Crypt_Token_KeyInfo_GetModulusData(ki);
+      modLen=GWEN_Crypt_Token_KeyInfo_GetModulusLen(ki);
+      nbits=modLen*8;
+      while(modLen && *modPtr==0) {
+	nbits-=8;
+	modLen--;
+        modPtr++;
+      }
+      if (modLen) {
+	uint8_t b=*modPtr;
+	int i;
+	uint8_t mask=0x80;
+
+	for (i=0; i<8; i++) {
+	  if (b & mask)
+	    break;
+	  nbits--;
+	  mask>>=1;
+	}
+      }
+
+      if (nbits==0) {
+	DBG_ERROR(GWEN_LOGDOMAIN, "Empty modulus");
+	GWEN_Buffer_free(tbuf);
+	return GWEN_ERROR_GENERIC;
+      }
+
+      md=GWEN_MDigest_Sha256_new();
+      rv=GWEN_Padd_VerifyPkcs1Pss((const uint8_t*) GWEN_Buffer_GetStart(tbuf),
+				  GWEN_Buffer_GetUsedBytes(tbuf),
+				  nbits,
+				  pInData, inLen,
+				  inLen,
+				  md);
+      GWEN_MDigest_free(md);
+      if (rv<0) {
+	DBG_ERROR(GWEN_LOGDOMAIN, "here (%d)", rv);
+	return rv;
+      }
     }
-    if (l!=inLen) {
-      DBG_ERROR(GWEN_LOGDOMAIN, "Signature length doesn't match");
-      GWEN_Buffer_free(tbuf);
-      return GWEN_ERROR_VERIFY;
-    }
-    if (memcmp(pInData, GWEN_Buffer_GetStart(tbuf), l)!=0) {
-      DBG_ERROR(GWEN_LOGDOMAIN, "Signature doesn't match:");
-      GWEN_Buffer_free(tbuf);
-      return GWEN_ERROR_VERIFY;
+    else {
+      rv=GWEN_Padd_UnapplyPaddAlgo(a, tbuf);
+      if (rv<0) {
+	DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+	GWEN_Buffer_free(tbuf);
+	return rv;
+      }
+      l=GWEN_Buffer_GetUsedBytes(tbuf);
+
+      if (l!=inLen) {
+	DBG_ERROR(GWEN_LOGDOMAIN, "Signature length doesn't match");
+	GWEN_Buffer_free(tbuf);
+	return GWEN_ERROR_VERIFY;
+      }
+      if (memcmp(pInData, GWEN_Buffer_GetStart(tbuf), l)!=0) {
+	DBG_ERROR(GWEN_LOGDOMAIN, "Signature doesn't match:");
+	GWEN_Buffer_free(tbuf);
+	return GWEN_ERROR_VERIFY;
+      }
     }
   }
   else {
@@ -1412,7 +1546,7 @@ GWEN_Crypt_TokenFile__Verify(GWEN_CRYPT_TOKEN *ct,
       return rv;
     }
 
-    /* sign with key */
+    /* verify with key */
     rv=GWEN_Crypt_Key_Verify(k,
 			     (const uint8_t*)GWEN_Buffer_GetStart(srcBuf),
 			     GWEN_Buffer_GetUsedBytes(srcBuf),
@@ -1666,6 +1800,7 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
   uint32_t klen;
   GWEN_CRYPT_TOKEN_KEYINFO *cki;
   GWEN_CRYPT_TOKEN_KEYINFO *ki;
+  int sizeInBits;
 
   assert(ct);
   lct=GWEN_INHERIT_GETDATA(GWEN_CRYPT_TOKEN, GWEN_CRYPT_TOKEN_FILE, ct);
@@ -1706,12 +1841,25 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
     i--;
   }
 
-  /* generate key pair */
-  rv=GWEN_Crypt_KeyRsa_GeneratePair(GWEN_Crypt_CryptAlgo_GetChunkSize(a),
-				    (GWEN_Crypt_Token_GetModes(ct) &
-				     GWEN_CRYPT_TOKEN_MODE_EXP_65537)?1:0,
-				    &pubKey,
-				    &secKey);
+  sizeInBits=GWEN_Crypt_CryptAlgo_GetKeySizeInBits(a);
+  if (sizeInBits>0) {
+    /* generate key pair with precise number of bits */
+    DBG_DEBUG(GWEN_LOGDOMAIN, "Creating key pair using %d bits", sizeInBits);
+    rv=GWEN_Crypt_KeyRsa_GeneratePair2(sizeInBits,
+				       (GWEN_Crypt_Token_GetModes(ct) &
+					GWEN_CRYPT_TOKEN_MODE_EXP_65537)?1:0,
+				       &pubKey,
+				       &secKey);
+  }
+  else {
+    /* generate key pair the old way, just using the chunksize */
+    DBG_INFO(GWEN_LOGDOMAIN, "Creating key pair using %d bytes", GWEN_Crypt_CryptAlgo_GetChunkSize(a));
+    rv=GWEN_Crypt_KeyRsa_GeneratePair(GWEN_Crypt_CryptAlgo_GetChunkSize(a),
+				      (GWEN_Crypt_Token_GetModes(ct) &
+				       GWEN_CRYPT_TOKEN_MODE_EXP_65537)?1:0,
+				      &pubKey,
+				      &secKey);
+  }
   if (rv) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
     GWEN_Gui_ProgressLog(gid, GWEN_LoggerLevel_Error,
@@ -1765,7 +1913,7 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
 
   if (keyNum==1) {
     if (GWEN_Crypt_Token_GetModes(ct) & GWEN_CRYPT_TOKEN_MODE_DIRECT_SIGN){
-      DBG_ERROR(0, "Adding mode \"direct sign\" to key");
+      DBG_DEBUG(GWEN_LOGDOMAIN, "Adding mode \"direct sign\" to key");
       GWEN_Crypt_KeyRsa_AddFlags(secKey, GWEN_CRYPT_KEYRSA_FLAGS_DIRECTSIGN);
     }
     GWEN_CTF_Context_SetLocalSignKey(ctx, secKey);
@@ -1795,7 +1943,7 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
   }
   else {
     if (GWEN_Crypt_Token_GetModes(ct) & GWEN_CRYPT_TOKEN_MODE_DIRECT_SIGN){
-      DBG_ERROR(0, "Adding mode \"direct sign\" to key");
+      DBG_DEBUG(GWEN_LOGDOMAIN, "Adding mode \"direct sign\" to key");
       GWEN_Crypt_KeyRsa_AddFlags(secKey, GWEN_CRYPT_KEYRSA_FLAGS_DIRECTSIGN);
     }
     GWEN_CTF_Context_SetLocalAuthKey(ctx, secKey);
@@ -1834,7 +1982,7 @@ GWEN_Crypt_TokenFile__GenerateKey(GWEN_CRYPT_TOKEN *ct,
 
 
 GWENHYWFAR_CB
-void GWEN_Crypt_TokenFile_freeData(void *bp, void *p) {
+void GWEN_Crypt_TokenFile_freeData(GWEN_UNUSED void *bp, void *p) {
   GWEN_CRYPT_TOKEN_FILE *lct;
 
   lct=(GWEN_CRYPT_TOKEN_FILE*) p;
