@@ -103,7 +103,7 @@ GWEN_GUI *GWEN_Gui_CGui_new() {
 
 
 
-void GWENHYWFAR_CB GWEN_Gui_CGui_FreeData(void *bp, void *p) {
+void GWENHYWFAR_CB GWEN_Gui_CGui_FreeData(GWEN_UNUSED void *bp, void *p) {
   GWEN_GUI_CGUI *cgui;
 
   cgui=(GWEN_GUI_CGUI*)p;
@@ -146,25 +146,31 @@ void GWEN_Gui_CGui_SetCharSet(GWEN_GUI *gui, const char *s) {
 
 
 int GWEN_Gui_CGui_GetIsNonInteractive(const GWEN_GUI *gui) {
-  GWEN_GUI_CGUI *cgui;
-
-  assert(gui);
-  cgui=GWEN_INHERIT_GETDATA(GWEN_GUI, GWEN_GUI_CGUI, gui);
-  assert(cgui);
-
-  return cgui->nonInteractive;
+  return GWEN_Gui_GetFlags(gui) & GWEN_GUI_FLAGS_NONINTERACTIVE;
 }
 
 
 
 void GWEN_Gui_CGui_SetIsNonInteractive(GWEN_GUI *gui, int i) {
-  GWEN_GUI_CGUI *cgui;
+  if (i)
+    GWEN_Gui_AddFlags(gui, GWEN_GUI_FLAGS_NONINTERACTIVE);
+  else
+    GWEN_Gui_SubFlags(gui, GWEN_GUI_FLAGS_NONINTERACTIVE);
+}
 
-  assert(gui);
-  cgui=GWEN_INHERIT_GETDATA(GWEN_GUI, GWEN_GUI_CGUI, gui);
-  assert(cgui);
 
-  cgui->nonInteractive=i;
+
+int GWEN_Gui_CGui_GetAcceptAllValidCerts(const GWEN_GUI *gui) {
+  return GWEN_Gui_GetFlags(gui) & GWEN_GUI_FLAGS_ACCEPTVALIDCERTS;
+}
+
+
+
+void GWEN_Gui_CGui_SetAcceptAllValidCerts(GWEN_GUI *gui, int i) {
+  if (i)
+    GWEN_Gui_AddFlags(gui, GWEN_GUI_FLAGS_ACCEPTVALIDCERTS);
+  else
+    GWEN_Gui_SubFlags(gui, GWEN_GUI_FLAGS_ACCEPTVALIDCERTS);
 }
 
 
@@ -343,7 +349,7 @@ char GWEN_Gui_CGui__readCharFromStdin(int waitFor) {
 
 
 
-int GWEN_Gui_CGui__input(GWEN_GUI *gui,
+int GWEN_Gui_CGui__input(GWEN_UNUSED GWEN_GUI *gui,
 			 uint32_t flags,
 			 char *buffer,
 			 int minLen,
@@ -483,7 +489,7 @@ int GWEN_Gui_CGui_MessageBox(GWEN_GUI *gui,
 			     const char *b1,
 			     const char *b2,
 			     const char *b3,
-			     uint32_t guiid) {
+			     GWEN_UNUSED uint32_t guiid) {
   GWEN_GUI_CGUI *cgui;
   GWEN_BUFFER *tbuf;
   int c;
@@ -495,7 +501,7 @@ int GWEN_Gui_CGui_MessageBox(GWEN_GUI *gui,
   tbuf=GWEN_Buffer_new(0, 256, 0, 1);
   GWEN_Gui_CGui_GetRawText(gui, text, tbuf);
 
-  if (cgui->nonInteractive) {
+  if (GWEN_Gui_GetFlags(gui) & GWEN_GUI_FLAGS_NONINTERACTIVE) {
     if (GWEN_GUI_MSG_FLAGS_SEVERITY_IS_DANGEROUS(flags)) {
       fprintf(stderr,
               "Got the following dangerous message:\n%s\n",
@@ -622,10 +628,10 @@ int GWEN_Gui_CGui_InputBox(GWEN_GUI *gui,
 
 
 uint32_t GWEN_Gui_CGui_ShowBox(GWEN_GUI *gui,
-			       uint32_t flags,
+			       GWEN_UNUSED uint32_t flags,
 			       const char *title,
 			       const char *text,
-			       uint32_t guiid) {
+			       GWEN_UNUSED uint32_t guiid) {
   GWEN_GUI_CGUI *cgui;
   GWEN_BUFFER *tbuf;
 
@@ -646,7 +652,7 @@ uint32_t GWEN_Gui_CGui_ShowBox(GWEN_GUI *gui,
 
 
 
-void GWEN_Gui_CGui_HideBox(GWEN_GUI *gui, uint32_t id) {
+void GWEN_Gui_CGui_HideBox(GWEN_GUI *gui, GWEN_UNUSED uint32_t id) {
   GWEN_GUI_CGUI *cgui;
 
   assert(gui);
@@ -663,7 +669,7 @@ uint32_t GWEN_Gui_CGui_ProgressStart(GWEN_GUI *gui,
 				     const char *title,
 				     const char *text,
 				     uint64_t total,
-				     uint32_t guiid) {
+				     GWEN_UNUSED uint32_t guiid) {
   GWEN_GUI_CGUI *cgui;
   GWEN_GUI_CPROGRESS *cp;
 
@@ -775,12 +781,12 @@ int GWEN_Gui_CGui_ProgressEnd(GWEN_GUI *gui,uint32_t id) {
 
 
 
-int GWEN_Gui_CGui_Print(GWEN_GUI *gui,
-			const char *docTitle,
-			const char *docType,
-			const char *descr,
-			const char *text,
-			uint32_t guiid) {
+int GWEN_Gui_CGui_Print(GWEN_UNUSED GWEN_GUI *gui,
+			GWEN_UNUSED const char *docTitle,
+			GWEN_UNUSED const char *docType,
+			GWEN_UNUSED const char *descr,
+			GWEN_UNUSED const char *text,
+			GWEN_UNUSED uint32_t guiid) {
   return GWEN_ERROR_NOT_SUPPORTED;
 }
 
@@ -845,12 +851,24 @@ int GWEN_Gui_CGui_CheckCert(GWEN_GUI *gui,
     return 0;
   }
 
-  if (cgui->nonInteractive) {
-    DBG_NOTICE(GWEN_LOGDOMAIN,
-	       "Automatically rejecting certificate [%s] (noninteractive)",
-	       hash);
-    GWEN_Buffer_free(hbuf);
-    return GWEN_ERROR_USER_ABORTED;
+  if (GWEN_Gui_GetFlags(gui) & GWEN_GUI_FLAGS_NONINTERACTIVE) {
+    uint32_t fl;
+
+    fl=GWEN_SslCertDescr_GetStatusFlags(cd);
+    if (fl==GWEN_SSL_CERT_FLAGS_OK && (GWEN_Gui_GetFlags(gui) & GWEN_GUI_FLAGS_ACCEPTVALIDCERTS)) {
+      DBG_NOTICE(GWEN_LOGDOMAIN,
+		 "Automatically accepting valid new certificate [%s]",
+		 hash);
+      GWEN_Buffer_free(hbuf);
+      return 0;
+    }
+    else {
+      DBG_NOTICE(GWEN_LOGDOMAIN,
+		 "Automatically rejecting certificate [%s] (noninteractive)",
+		 hash);
+      GWEN_Buffer_free(hbuf);
+      return GWEN_ERROR_USER_ABORTED;
+    }
   }
 
   if (cgui->checkCertFn) {
@@ -875,7 +893,7 @@ int GWEN_Gui_CGui_SetPasswordStatus(GWEN_GUI *gui,
 				    const char *token,
 				    const char *pin,
 				    GWEN_GUI_PASSWORD_STATUS status,
-				    uint32_t guiid) {
+				    GWEN_UNUSED uint32_t guiid) {
   GWEN_GUI_CGUI *cgui;
 
   assert(gui);
@@ -957,7 +975,7 @@ int GWEN_Gui_CGui_GetPassword(GWEN_GUI *gui,
       }
     }
 
-    if (cgui->nonInteractive) {
+    if (GWEN_Gui_GetFlags(gui) & GWEN_GUI_FLAGS_NONINTERACTIVE) {
       DBG_ERROR(GWEN_LOGDOMAIN,
 		"Password for [%s] missing in noninteractive mode, "
 		"aborting", token);
