@@ -57,6 +57,7 @@
 
 #include "io/bufferedio_l.h"
 #include "parser/dbio_l.h"
+#include "parser/configmgr_l.h"
 #include "crypt3/cryptkey_l.h"
 #include "crypttoken/ctplugin_l.h"
 #include "iolayer/iomanager_l.h"
@@ -251,6 +252,10 @@ int GWEN_Init() {
       return err;
     DBG_DEBUG(GWEN_LOGDOMAIN, "Initializing DataBase IO module");
     err=GWEN_DBIO_ModuleInit();
+    if (err)
+      return err;
+    DBG_DEBUG(GWEN_LOGDOMAIN, "Initializing ConfigMgr module");
+    err=GWEN_ConfigMgr_ModuleInit();
     if (err)
       return err;
     DBG_DEBUG(GWEN_LOGDOMAIN, "Initializing CryptToken2 module");
@@ -287,6 +292,12 @@ int GWEN_Fini() {
       DBG_ERROR(GWEN_LOGDOMAIN, "GWEN_Fini: "
 		"Could not deinitialze module CryptToken2");
     }
+    lerr=GWEN_ConfigMgr_ModuleFini();
+    if (lerr) {
+      err=lerr;
+      DBG_ERROR(GWEN_LOGDOMAIN, "GWEN_Fini: "
+		"Could not deinitialze module ConfigMgr");
+    }
     lerr=GWEN_DBIO_ModuleFini();
     if (lerr) {
       err=lerr;
@@ -452,27 +463,3 @@ int GWEN__GetValueFromWinReg(const char *keyPath,
 #endif /* 0 */
 
 
-
-/** Construct the gwenhywfar_plugindir directory path with runtime
-    lookup of the initial $libdir. */
-char *GWEN__get_plugindir (const char *default_dir)
-{
-  char *result, *libdir, *pkglibdir, *tmp;
-
-  libdir = br_find_lib_dir (NULL);
-  if (libdir == (char *) NULL)
-    /* BinReloc not initialized, use default hard-coded path. */
-    return strdup (default_dir);
-
-  pkglibdir = br_build_path (libdir, PACKAGE); /* defined in config.h */
-  tmp = br_build_path (pkglibdir, "plugins");
-  result = br_build_path (tmp, GWENHYWFAR_SO_EFFECTIVE_STR);
-
-  free (libdir);
-  free (pkglibdir);
-  free (tmp);
-
-  return result;
-}
-
-
