@@ -30,7 +30,14 @@
 #endif
 
 #include "directory_p.h"
-#include "binreloc.h"
+
+#ifdef OS_DARWIN
+# ifdef ENABLE_LOCAL_INSTALL
+#   include <CoreFoundation/CFBundle.h>
+# endif
+#else
+# include "binreloc.h"
+#endif
 
 #include <stdlib.h>
 #include <assert.h>
@@ -70,8 +77,11 @@ int GWEN_Directory_Open(GWEN_DIRECTORY *d, const char *n){
   assert(d);
 
   d->handle=opendir(n);
-  if (d->handle==0)
-    return 1;
+  if (d->handle==0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "opendir(%s): %s",
+             n, strerror(errno));
+    return GWEN_ERROR_NOT_FOUND;
+  }
   return 0;
 }
 
@@ -100,19 +110,19 @@ int GWEN_Directory_Read(GWEN_DIRECTORY *d,
   if (de) {
     if (len<strlen(de->d_name)+1) {
       DBG_ERROR(GWEN_LOGDOMAIN, "Buffer too small");
-      return 1;
+      return GWEN_ERROR_BUFFER_OVERFLOW;
     }
     strcpy(buffer,de->d_name);
     return 0;
   }
-  return 1;
+  return GWEN_ERROR_NOT_FOUND;
 }
 
 
 int GWEN_Directory_Rewind(GWEN_DIRECTORY *d){
   assert(d);
   if (d->handle==0)
-    return 1;
+    return GWEN_ERROR_INVALID;
   rewinddir(d->handle);
   return 0;
 }
@@ -194,7 +204,19 @@ int GWEN_Directory_CreatePublic(const char *path){
 int GWEN_Directory_GetPrefixDirectory(char *buffer, unsigned int size){
   char *exeDir;
 
+#if defined(OS_DARWIN) && defined(ENABLE_LOCAL_INSTALL)
+  CFBundleRef mainBundle = CFBundleGetMainBundle() ;
+  CFURLRef resourcesURL = CFBundleCopyResourcesDirectoryURL(mainBundle);
+  CFStringRef resourcesPath = CFURLCopyFileSystemPath(resourcesURL, kCFURLPOSIXPathStyle) ;
+  CFIndex maxPathSize = CFStringGetMaximumSizeOfFileSystemRepresentation(resourcesPath);
+  if ((exeDir = malloc(maxPathSize * sizeof(char)))) {
+    CFStringGetFileSystemRepresentation(resourcesPath, exeDir , maxPathSize);
+  }
+  CFRelease(resourcesPath);
+  CFRelease(resourcesURL);
+#else
   exeDir=br_find_prefix(NULL);
+#endif
   if (exeDir==(char*)NULL) {
     DBG_INFO(GWEN_LOGDOMAIN,
 	     "Unable to determine exe folder");
